////////////////////////////////////////////////////////////////////////////////
//
//  Licensed to the Apache Software Foundation (ASF) under one or more
//  contributor license agreements.  See the NOTICE file distributed with
//  this work for additional information regarding copyright ownership.
//  The ASF licenses this file to You under the Apache License, Version 2.0
//  (the "License"); you may not use this file except in compliance with
//  the License.  You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.
//
////////////////////////////////////////////////////////////////////////////////

package mx.charts.series.renderData
{

import mx.charts.chartClasses.RenderData;

/**
 *  Represents all the information needed by the HLOCSeries and CandlestickSeries objects to render.  
 *  
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 */
public class HLOCSeriesRenderData extends RenderData
{
    include "../../../core/Version.as";

    //--------------------------------------------------------------------------
    //
    //  Constructor
    //
    //--------------------------------------------------------------------------

	/**
	 *  Constructor.
	 *
	 *	@param	cache	The list of HLOCSeriesItem objects representing the items in the data provider.
	 *	@param	filteredCache	The list of HLOCSeriesItem objects representing the items in the data provider that remain after filtering.
	 *	@param	renderedHalfWidth	Half the width of an item, in pixels.
	 *	@param	renderedXOffset		The offset of each item from its x value, in pixels.
	 *  
	 *  @langversion 3.0
	 *  @playerversion Flash 9
	 *  @playerversion AIR 1.1
	 *  @productversion Flex 3
	 */
	public function  HLOCSeriesRenderData(cache:Array /* of HLOCSeriesItem */ = null,
										  filteredCache:Array /* of HLOCSeriesItem */ = null,
										  renderedHalfWidth:Number = 0,
										  renderedXOffset:Number = 0) 
	{
		super(cache, filteredCache);

		this.renderedHalfWidth = renderedHalfWidth;
		this.renderedXOffset = renderedXOffset;
	}

    //--------------------------------------------------------------------------
    //
    //  Properties
    //
    //--------------------------------------------------------------------------

    //----------------------------------
	//  renderedHalfWidth
    //----------------------------------

	[Inspectable(environment="none")]

	/**
	 *  Half the width of an item, in pixels.
	 *  
	 *  @langversion 3.0
	 *  @playerversion Flash 9
	 *  @playerversion AIR 1.1
	 *  @productversion Flex 3
	 */
	public var renderedHalfWidth:Number;

    //----------------------------------
	//  renderedXOffset
    //----------------------------------

	[Inspectable(environment="none")]

	/**
	 *  The offset of each item from its x value, in pixels.
	 *  
	 *  @langversion 3.0
	 *  @playerversion Flash 9
	 *  @playerversion AIR 1.1
	 *  @productversion Flex 3
	 */
	public var renderedXOffset:Number;

    //--------------------------------------------------------------------------
    //
    //  Methods
    //
    //--------------------------------------------------------------------------

	/**
	 *  @private
	 */
	override public function clone():RenderData
	{
		return new HLOCSeriesRenderData(cache, filteredCache,
										renderedHalfWidth, renderedXOffset);
	}
}

}